# OCS Powershell example Functions. Copyright Microsoft Corporation 2007-8
# These Functions are supplied with no warranty of any kind. 
# You may modify the powershell code and reuse it as you see fit 

# The code in this file was written by James O'Neill (http://blogs.technet.com/jamesone)
# With help from Michael Hefner, Sasa Juratovic, Rick Kingslan, Rui Maximo and Mitch Tulloch, 

# More information may appear on the CommunicationServer Team blog (http://communicationsserverteam.com/) 


Function Choose-List
{Param ($InputObject, $Property, [Switch]$multiple)
 $Global:counter=-1
 $Property=@(@{Label="ID"; Expression={ ($global:Counter++) }}) + $Property
 if ($inputObject -is [Array]) {
     $InputObject | format-table -autosize -property $Property | out-host
     if ($multiple) { $InputObject[ [int[]](Read-Host "Which one(s) ?").Split(",")] }
     else           { $InputObject[        (Read-Host "Which one ?")              ] }}
 else {$inputObject}
}


Function Get-ScriptPath
{ split-path $myInvocation.scriptName }


Function Get-OCSInstalledService {Param($Server=".") ;  Get-WmiObject -query "select * from MSFT_SIPServerInstalledComponentData"-computername $server | sort-object -property name }


Function Get-OCSSIPDomain 
{Param($name="%", $Server=".") 
 $name=$name.replace("*","%")
 Get-WmiObject -class MSFT_SIPDomainData -filter "address like'$name'" -computername $server 
}


Function New-OCSSIPDomain
{Param($Domain,$server=".")
 if ($domain -is [array]) {$domain | foreach-object {New-OCSSIPDomain -Domain $_ -Server $server} }
 if ($domain -is [String]) {
    $Newdomain=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPDomainData").createinstance()
    $newdomain.Address = $domain
    $newdomain.Authoritative = $false
    $newdomain.DefaultDomain = $false
    $newdomain.put() | out-null }
}


Function Remove-OCSSipDomain 
{Param($Domain,$server=".")
 if ($domain -is [array]) {$domain | foreach-object {Remove-OCSSIPDomain -Domain $_ -Server $server} }
 if ($domain -is [String]) {(Get-WmiObject -computerName $Server -query "select * from MSFT_SIPDomainData where address='$domain' ").delete() | out-null }
}

Function Get-OCSPool 
{Param($name="%", $Server=".")
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPPoolSetting where PoolDisplayname like '$name'" -computername $server | sort-object -Property PoolDisplayName}


Function Choose-OCSPool {Param($Server=".") ; choose-list -input (Get-OCSPool -server $server) -property PoolFqdn }


Function Get-OCSADContainer
{#runs against default AD server
 $searcher= New-Object directoryServices.DirectorySearcher([adsi]('LDAP://'+([adsi]'LDAP://RootDse').RootDomainNamingContext ))
 $searcher.filter="(objectclass=msRTCSIP-Service)"
 [string]$global:OCSADContainer=$searcher.FindOne().path
 if ($OCSADContainer -eq '') 
    {$searcher= New-Object directoryServices.DirectorySearcher([adsi]('LDAP://'+([adsi]'LDAP://RootDse').configurationNamingContext))
     $searcher.filter="(objectclass=msRTCSIP-Service)"
     [string]$global:OCSADContainer=$searcher.FindOne().path }
 $OCSADContainer
}


Function Get-OCSSchemaVersion
{#runs against default AD server
 $RTCSchemaContext = [adsi]("LDAP://CN=ms-RTC-SIP-SchemaVersion," + ([adsi]"LDAP://rootDSE").schemaNamingContext)
 if (($RTCSchemaContext.rangeLower -ge 2 -and $RTCSchemaContext.rangeLower -le 20) -and $RTCSchemaContext.rangeUpper -eq 1008)
    {'Active Directory schema supports Office Communications Server 2007 R2 servers.'}
 elseif (($RTCSchemaContext.rangeLower -ge 13 -and $RTCSchemaContext.rangeLower -le 20) -and $RTCSchemaContext.rangeUpper -eq 1007)
    {'Active Directory schema supports Office Communications Server 2007 RTM servers.'}
 elseif (($RTCSchemaContext.rangeLower -ge 2 -and $RTCSchemaContext.rangeLower -le 6) -and $RTCSchemaContext.rangeUpper -eq 1006)
    {'Active Directory schema supports Live Communications Server 2005 Service Pack 1 servers.'}
 elseif ($RTCSchemaContext.rangeLower -eq 1005 -and $RTCSchemaContext.rangeUpper -eq 1006)
      {'Active Directory schema supports Live Communications Server 2005 RTM servers.'}
 else {'Active Directory schema extensions are not present or schema unknown.' }
}


Function Get-OCSWarningEvent {Get-EventLog -LogName 'Office Communications Server' | where-object {$_.EntryType -eq 'Warning' } |   format-table -Wrap -AutoSize -property TimeGenerated, eventID ,  message }
# Lots of scope for you to define your own Where conditions and formatting 
# Note the Get-EventLog only works on the local server

Function Get-OCSErrorEvent {Get-EventLog -LogName 'Office Communications Server' | where-object {$_.EntryType -eq 'Error' } |  format-table -Wrap -AutoSize -property TimeGenerated, eventID ,  message }
# Lots of scope for you to define your own Where conditions and formatting 
# Note the Get-EventLog only works on the local server
 

Function Get-OCSUser
{Param([String]$URI, $Condition, $Server=".")
 if (($URI -eq "%") -or ($URI="*")) {$uri=$null}
 IF ($URI) { if ($URI.Substring(0,4) -ne 'SIP:') {$URI = "SIP:$URI"}
             if ($Condition) {$condition += "And (PrimaryURI='$URI') "} else {$condition = "PrimaryURI='$URI' "} }
 if ($condition) {$WQL = "Select * from msft_sipesusersetting where $Condition" } else {$WQL="Select * from msft_sipesusersetting" }
 Get-wmiObject -query $wql -ComputerName $Server 
}
#Tip: if you're writing conditions use the field names displayed by Get-OCSUser | gm 


Function Remove-OCSUser
#You can use this as template for a more general delete. But great care is needed
{Param([String]$URI=$(throw "You must specify a URI") , $Server=".")
 Get-OCSUser -uri $URI  -Server $Server | foreach-object {$_.Delete()}
}


Function Get-OCSUserDetail
{Param($uri, $Condition, $Server=".")
 #A more complex way to display user information. 
 $pools=Get-OCSPool -Server $Server
 $Mtgpolicies=Get-OCSMeetingPolicy -Server $Server
 $UCPolicies=Get-OCSUCPolicy -Server $Server
 Get-OCSUser -URI $URI -condition $condition -Server $Server | sort-object -property DisplayName |
   Format-List -Property DisplayName, @{label='URI' ; expression={$_.PrimaryURI }},
   @{label='Server'; expression={$Dn=$_.homeserverDN
          $pools | where-object {$_.pooldn -eq $dn} | foreach {$_.poolFqdn} }},
   @{label='Enabled'; expression={$_.Enabled}},
   @{label="Enhanced presence"; expression={$_.EnabledForEnhancedPresence}},
   @{label="Remote Access"; expression={$_.EnabledForInternetAccess}},
   @{label="Federation"; expression={$_.EnabledForFederation}},
   @{label='PIC enabled'; expression={$_.PublicNetworkEnabled}}, 
   @{label='Voice Enabled'; expression={$_.UCEnabled}},
   @{label='Tel: URI'; expression={$_.LineURI}},
   @{label='UC Policy'; expression={$UCPol=$_.UCPolicy
   $UCpolicies | where-object {$_.PolicyDN -eq $UCPol} | foreach {$_.name} }},
   @{label='Meeting Policy'; expression={$Pol=$_.MeetingPolicy
          $MTGpolicies | where-object {$_PolicyDN -eq $pol} | foreach {$_.name} }}
}
#EXAMPLE Get-OCSUserDetail -condition "PrimaryURI like 'SIP:S%'" 


Function Get-OCSPICUserCount  
{Param($Condition, $Server=".")
 Get-OCSUser -condition "PublicNetworkEnabled = 'TRUE'" -server $Server | measure-object | Select-Object -Property count }
#There are plenty of selection criteria you might have for choosing users to count


Function Get-ADUser
{Param($Container, $filter)
 #runs against default AD server
 if ($Container -eq $null) {$searcher= New-Object directoryServices.DirectorySearcher([adsi]('GC://'+([adsi]'LDAP://RootDse').RootDomainNamingContext ))}
 else {$searcher= New-Object directoryServices.DirectorySearcher([adsi]("GC://$Container"))}
 if ($Filter -eq $null) {$searcher.filter="(objectclass=user)"}
 else {$searcher.filter="(&(objectclass=user)($filter))"}
 $searcher.Findall()
}


Function Export-ADUser
{Param([String]$Path=$(throw "You must specify a File") , $Container, $filter)
 Get-ADUser $Container, $filter| 
 select-object @{Name="distinguishedname"; Expression = {$_.Properties.item("distinguishedname")}},
  @{Name="DisplayName"; Expression = {$_.Properties.item("DisplayName")}},
  @{Name="msrtcsip-userenabled"; Expression = {$_.Properties.item("msrtcsip-userenabled")}},
  @{Name="msrtcsip-primaryuseraddress"; Expression = {$_.Properties.item("msrtcsip-primaryuseraddress")}},
  @{Name="msrtcsip-primaryhomeserver"; Expression = {$_.Properties.item("msrtcsip-primaryhomeserver")}},
  @{Name="telephonenumber"; Expression = {$_.Properties.item("telephonenumber")}},
  @{Name="Mail"; Expression = {$_.Properties.item("Mail")}} | Export-csv -path $path
}


Filter New-OCSuser
{Param($URI, $User , $HomeServer, $Server=".")
 if ((-not $user) -and $_) {
        if ($_ -is [System.DirectoryServices.SearchResult])  { $user = $_.Properties.item("distinguishedName")}
        if ($_ -is [System.DirectoryServices.DirectoryEntry]){$User = $_.distinguishedName}  }
 if ((-not $URI) -and $user ) {$uri=[String]([adsi]"GC://$user").mail}
 if ($URI.Substring(0,4).toupper() -ne 'SIP:') {$URI = "SIP:$URI"}
 if ($homeServer -is [System.Management.ManagementObject]) {$homeServer=$homeServer.poolDN}
 if ($URI -and $user -and $homeserver) {
    if ((Get-OCSuser -Condition "PrimaryURI='$URI'") -eq $null)
         { if (-not (Get-OCSSipDomain $uri.split('@')[-1])) {Write-warning "$URI is not in a supported domain. User was enabled anyway."}
           $NewUser=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPESUserSetting").createinstance()
           $newUser.enabled = $true
           $newuser.PrimaryURI = $URI
           $newUser.UserDN = $User
           $newUser.HomeServerDN = $HomeServer
           $newUser.put() | out-null
           $newUserObject=(Get-OCSuser -uri  $URI)
           if ($newuserObject -eq $null) {write-Host "$URI : Creation failed"} else {$newuserobject}
         }
    else {write-Host "$URI already exists"} }
 $URI=$null
 $User=$null
}


Filter Update-OCSUser
{Param($Condition,$URI,$UserDN, $AllowOrganizeMeetingWithAnonymousParticipants, $ArchiveFederatedCommunications, $ArchiveInternalCommunications, 
 $EnabledForEnhancedPresence, $EnabledForFederation, $EnabledForInternetAccess, $HomeServerDN, $UCEnabled, $LineURI, $MeetingPolicy, 
 $PublicNetworkEnabled, $RemoteCallControlTelephonyEnabled, $UCPolicy, $Server=".") 
  if ($userDN ) {$uri=[String]([adsi]"GC://$userDN")."msRTCSIP-PrimaryUserAddress" }
  if ($_ -is [System.DirectoryServices.SearchResult])  {$uri=$_.Properties.item("msRTCSIP-PrimaryUserAddress")}
  if ($_ -is [System.DirectoryServices.DirectoryEntry]){$uri=$_."msRTCSIP-PrimaryUserAddress" } 
  if ($_ -is [System.Management.ManagementObject]) 
      {$users = $_ }
  else
      {$users=(Get-OCSUser -condition $condition -uri $uri -Server $server) }
  $users | foreach-object { 
        $changed = $false
               if (( $EnabledForInternetAccess -ne $null) -and ( $EnabledForInternetAccess -ne $_.EnabledForInternetAccess)) 
                        {$_.EnabledForInternetAccess = ($EnabledForInternetAccess -eq $true)
                          $changed = $true}
               if (( $RemoteCallControlTelephonyEnabled -ne $null) -and ( $RemoteCallControlTelephonyEnabled -ne $_.RemoteCallControlTelephonyEnabled)) 
                        {$_.RemoteCallControlTelephonyEnabled= ($RemoteCallControlTelephonyEnabled -eq $true)
                          $changed = $true}
               if (( $HomeServerDN -ne $null) -and ( $HomeServerDN -ne $_.HomeServerDN)) 
                        {$_.HomeServerDN= $HomeServerDN
                          $changed = $true}
               if (( $EnabledForFederation -ne $null) -and ( $EnabledForFederation -ne $_.EnabledForFederation)) 
                        {$_.EnabledForFederation = ($EnabledForFederation -eq $true)
                          $changed = $true}
	       if (( $PublicNetworkEnabled -ne $null) -and ( $PublicNetworkEnabled -ne $_.PublicNetworkEnabled)) 
                        {$_.PublicNetworkEnabled = ($PublicNetworkEnabled -eq $true)
                          $changed = $true}
               if (( $MeetingPolicy -ne $null) -and ( $MeetingPolicy -ne $_.MeetingPolicy)) 
                        {$_.MeetingPolicy= $MeetingPolicy
                          $changed = $true}
               if (( $UCPolicy -ne $null) -and( $UCPolicy -ne $_.UCPolicy)) 
                        {$_.UCPolicy = $UCPolicy
                          $changed = $true}
               #Can't un-set Enhanced Presence so only set it if isn't set already
               if (( $EnabledForEnhancedPresence -eq $true) -and ($_.EnabledForEnhancedPresence -eq $false)) 
                        {$_.EnabledForEnhancedPresence= $true
                          $changed = $true}
               if (( $UCEnabled -ne $null) -and ( $UCEnabled -ne $_.UCEnabled)) 
                        {If ( $EnabledForEnhancedPresence -eq $true) {$_.UCEnabled = ($UCEnabled -eq $true) 
                          $changed = $true}
			 ELSE {Write-output " UC requires Enhanced presence to be enabled first"}}
               if (( $LineURI -ne $null) -and ( $LineURI -ne $_.LineURI)) 
                        {$_.LineURI = $LineURI
                          $changed = $true}
               if (( $AllowOrganizeMeetingWithAnonymousParticipants -ne $null) -and ( $AllowOrganizeMeetingWithAnonymousParticipants -ne $_.AllowOrganizeMeetingWithAnonymousParticipants)) 
                        {$_.AllowOrganizeMeetingWithAnonymousParticipants = ($AllowOrganizeMeetingWithAnonymousParticipants -eq $true)
                          $changed = $true}
               if (( $ArchiveFederatedCommunications -ne $null) -and ( $ArchiveFederatedCommunications-ne $_.ArchiveFederatedCommunications)) 
                        {$_.ArchiveFederatedCommunications = ($ArchiveFederatedCommunications -eq $true)
                          $changed = $true}
               if (( $ArchiveInternalCommunications -ne $null) -and ( $ArchiveInternalCommunications -ne $_.ArchiveInternalCommunications)) 
                        {$_.ArchiveInternalCommunications= ($ArchiveInternalCommunications -eq $true)
                          $changed = $true}
               if ( $changed) 
                        { 
   ('  ' + $_.PrimaryURI + ' Updated')
	$_.put() | out-null }
        else    {($_.PrimaryURI + ' Unchanged')}
         }
 $users=$null
}


Function Export-OCSUser
{Param([String]$Path=$(throw "You must specify a File") , $Condition , $Server=".")
#IPPBXSoftPhoneRoutingEnabled , LineServerURI not included
 Get-OCSUser -condition $Condition -Server $Server |
 select-object -property PrimaryURI, UserDN, homeserverDN, Enabled, EnabledForEnhancedPresence, EnabledForInternetAccess,
       EnabledForFederation, PublicNetworkEnabled ,UCEnabled, UCPolicy, MeetingPolicy, AllowOrganizeMeetingWithAnonymousParticipants,
       ArchiveFederatedCommunications, ArchiveInternalCommunications, LineURI, RemoteCallControlTelephonyEnabled | Export-csv -path $path
}


Function Import-OCSUser
{Param([String]$Path=$(throw "You must specify a File"), $server=".")
 Import-csv $path | foreach-object { 
   $user = Get-OCSUser -Uri $_.primaryURI -Server $Server
   if (($user -eq $null) -and ($_.enabled)) {$user=New-OCSuser -URI $_.PrimaryURI -UserDN $_.UserDN -HomeServerDN $_.homeserverDN}
   if (($user -ne $null) -and ($_.enabled -eq $false)) 
         {$user.Delete()
         ($_.PrimaryURI + ' deleted.')}
   elseif ($user -ne $null) {$user | Update-OCSUSer -AllowOrganizeMeetingWithAnonymousParticipants $_.AllowOrganizeMeetingWithAnonymousParticipants `
        -ArchiveFederatedCommunications $_.ArchiveFederatedCommunications -ArchiveInternalCommunications $_.ArchiveInternalCommunications  `
        -EnabledForEnhancedPresence $_.EnabledForEnhancedPresence  -EnabledForFederation $_.EnabledForFederation -EnabledForInternetAccess $_.EnabledForInternetAccess `
        -HomeServerDN $_.HomeServerDN -UCEnabled $_.UCEnabled -LineURI $_.LineURI -MeetingPolicy $_.MeetingPolicy -PublicNetworkEnabled $_.PublicNetworkEnabled`
        -RemoteCallControlTelephonyEnabled $_.RemoteCallControlTelephonyEnabled -UCPolicy $_.UCPolicy  }}
}


Function Get-OCSMeetingPolicy 
{Param($name="%", $Server=".") 
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPGlobalMeetingPolicyData where name like '$name'" -computername $server | sort-object -property name}


Function New-OCSMeetingPolicy
{Param($Name=$(throw "You must specify a name for the policy"), $Server=".", $MeetingSize=10, $AllowAppSharingForExternalMeeting='None', 
       $AllowIPAudio=$false, $AllowIPVideo=$false, $AllowPresenterToDelegateRecording=$false, $AllowPresenterToRecord=$false, 
       $ColorDepth='High colors', $EnableAppDesktopSharing=$false, $EnableDataCollaboration=$false, $RetainPPTForExternalMeeting=$false)
 if (Get-OCSMeetingPolicy -name $name -server $Server) {return $name + " already exists"}
 else {$instance=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPGlobalMeetingPolicyData").createinstance()
       $instance.Name=$name
       $instance.DefaultPolicy=$false
       $instance.MeetingSize=$meetingSize 
       $instance.AllowAppSharingForExternalMeeting=$AllowAppSharingForExternalMeeting
       $instance.AllowIPAudio=$AllowIPAudio
       $instance.AllowIPVideo=$AllowIPVideo 
       $instance.AllowPresenterToDelegateRecording=$AllowPresenterToDelegateRecording
       $instance.AllowPresenterToRecord=$AllowPresenterToRecord
       $instance.ColorDepth=$colorDepth              
       #'Gray shades', '256' , 'High colors' , 'True Colors'         
       $instance.EnableAppDesktopSharing=$EnableAppDesktopSharing 
       $instance.EnableDataCollaboration=$EnableDataCollaboration
       $instance.RetainPPTForExternalMeeting=$RetainPPTForExternalMeeting
       $instance.Type='meeting'
       $instance.put() | out-null}
}   


Filter Update-OCSMeetingPolicy
{Param($Name, $Server=".", $MeetingSize, $AllowAppSharingForExternalMeeting,  $AllowIPAudio, $AllowIPVideo, $AllowPresenterToDelegateRecording, 
       $AllowPresenterToRecord, $ColorDepth, $EnableAppDesktopSharing, $EnableDataCollaboration, $RetainPPTForExternalMeeting)
 if ($name) {$instance=(Get-OCSMeetingPolicy -name $name -server $Server)} else {$instance=$_}
 if ($instance) {
     if ($meetingSize -ne $null)  {$instance.MeetingSize=$meetingSize}
     if ($AllowAppSharingForExternalMeeting -ne $null) {$instance.AllowAppSharingForExternalMeeting=$AllowAppSharingForExternalMeeting}
     if ($AllowIPAudio -ne $null) {$instance.AllowIPAudio=$AllowIPAudio}
     If ($AllowIPVideo -ne $null) {$instance.AllowIPVideo=$AllowIPVideo} 
     if ($AllowPresenterToDelegateRecording -ne $null) {$instance.AllowPresenterToDelegateRecording=$AllowPresenterToDelegateRecording}
     if ($AllowPresenterToRecord -ne $null) {$instance.AllowPresenterToRecord=$AllowPresenterToRecord}
     if ($colorDepth  -ne $null)  {$instance.ColorDepth=$colorDepth}
     #'Gray shades', '256' , 'High colors' , 'True Colors'         
     if ($EnableAppDesktopSharing -ne $null) {$instance.EnableAppDesktopSharing=$EnableAppDesktopSharing }
     if ($EnableDataCollaboration -ne $null) {$instance.EnableDataCollaboration=$EnableDataCollaboration }
     if ($RetainPPTForExternalMeeting -ne $null) {$instance.RetainPPTForExternalMeeting=$RetainPPTForExternalMeeting}
     $instance.put() | out-null}
 else {"$name doesn't exist"}
 $Instance=$null
}   


Function Remove-OCSMeetingPolicy 
{Param($Name=$(throw "You must specify a name for the policy"), $server="." )
 (Get-OCSMeetingPolicy -name $name -server $Server).delete() | out-null }


Function Choose-OCSMeetingPolicy {Param($Server=".") ; choose-list -input (Get-OCSMeetingPolicy -server $server) -property name }

                   
Function Get-OCSUCPolicy 
{Param($name="%", $Server=".") 
 $name=$name.Replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPGlobalUCPolicyData where name like '$name' " -computername $server | sort-object -property Name }


Function New-OCSUCPolicy
{Param($Name=$(throw "You must specify a name for the policy"), [switch]$AllowSimultaneousRinging, $Server=".")
 if (Get-OCSUCPolicy -name $name -Server $Server) {return $name + " already exists"}
 else {$instance=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPGlobalUCPolicyData").createinstance()
       $instance.defaultPolicy=$false
       $instance.AllowSimultaneousRinging=$AllowSimultaneousRinging.ispresent
       $instance.Name=$name
       $instance.Type='uc'
       $instance.put() | out-null}
}


Filter Update-OCSUCPolicy {Param($Name, $AllowSimultaneousRinging, $defaultPolicy, $Server=".")
 if ($name) {$instance=Get-OCSUCPolicy -name $name -Server $Server} else {$instance=$_}
 if ($instance) {
       if ($defaultPolicy -ne $null) {$instance.defaultPolicy=$defaultPolicy}
       if ($AllowSimultaneousRinging -ne $null) {$instance.AllowSimultaneousRinging=$AllowSimultaneousRinging}
       $instance.put() | out-null}
 else {return $name + " Not found"}
 $instance=$null
}


Function Remove-OCSUCPolicy 
{Param([String]$Name=$(throw "You must specify a name for the Policy"), $Server=".")
 (Get-OCSUCPolicy -name $name -server $Server).delete() | out-null }


Function Choose-OCSUCPolicy {Param($Server=".") ; choose-list -input (Get-OCSUCPolicy -server $server) -property name }


Function Get-OCSNormalizationRule 
{Param($name="%", $Server=".")
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPLocalNormalizationRuleData where name like '$name' " -computername $server | sort-object -property Name }


Function Choose-OCSNormalizationRule {Param($Server=".") ; choose-list -input (Get-OCSNormalizationRule -server $server) -property name,description,pattern,translation -multi }


Function New-OCSNormalizationRule 
{Param( $Name=$(throw "You must specify a name for the rule") , $pattern=$(throw "You must specify a Pattern")  , $translation=$(throw "You must specify a translation rule"), $Description ,  $server="." )
 if (Get-OCSNormalizationRule -name $name -Server $Server) {return $name + " already exists"}
 else {$oInstance=([wmiclass]"\\$server\root\cimv2:MSFT_SIPLocalNormalizationRuleData").createinstance()
       $oInstance.name = $Name
       $oInstance.Description=$Description 
       $oInstance.Pattern=$pattern
       $oInstance.Translation=$translation
       $oinstance.put() | Out-Null }
}

#Note the pattern and translation rule will contain  $ signs, either wrap the terms in SINGLE quotes or escape the sign as `$ to stop powershell trying to expand it to a variable


Filter Remove-OCSNormalizationRule 
{Param($Rule, $Server=".")
 if ($rule -eq $null) {$rule = $_}
 if ($rule -is [String]) {$rule= (Get-OCSNormalizationRule  -name $Rule -Server $server) }
 if ($rule -is [System.Management.ManagementObject] ) { 
    Remove-OCSNormalizationRuleFromOCSLocationProfile $rule
    $rule.delete()  }
 $rule = $null
}


Filter Update-OCSNormalizationRule 
{Param($Name,  $Description , $pattern, $translation, $server=".")
 if ($name) {$oInstance=Get-OCSNormalizationRule  -name $name -Server $server } else {$oInstance=$_}
 if ($oInstance) {if ($description) { $oInstance.Description=$Description }
                  if ($Pattern)     { $oInstance.Pattern=$pattern}
                  if ($Translation) { $oInstance.Translation=$translation}
                   $oinstance.put() | Out-Null }
 else {return $name + " Not found"}
 $oInstance=$null
}


Function Get-OCSLocationProfile 
{Param($name="%", $Server=".") 
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPLocationProfileData where name like '$name' " -computername $server | sort-object -property Name }


Function Choose-OCSLocationProfile 
{Param($Server=".") ; choose-list -input (Get-OCSLocationProfile -server $server) -property name,description -multi} 


Function Remove-OCSLocationProfile  
{Param([String]$Name=$(throw "You must specify a name for the Profile"), $Server=".") 
 (Get-OCSLocationProfile -Server $Server -name $name).delete() }


Function New-OCSLocationProfile
{Param([String]$Name=$(throw "You must specify a name for the location") , $description=$(throw "You must specify a name for the location"), $rules=$(throw "You must specify one or more rules for the location profile"),
       [boolean]$OptimizeDeviceDialing=$false, $ExternalAccessPrefix, $Server=".")
 if ($rules -is [String]) {$rules = (Get-OCSNormalizationRule -name $rules) } 
 if (Get-OCSLocationProfile -name $name -Server $Server) {return $name + " already exists"}
 else {$oInstance=([wmiclass]"\\$server\root\cimv2:MSFT_SIPLocationProfileData").createinstance()
       $oInstance.Name=$name
       $oInstance.description=$description
       $oInstance.OptimizeDeviceDialing = $OptimizeDeviceDialing
       if ($ExternalAccessPrefix) {$oInstance.ExternalAccessPrefix=$ExternalAccessPrefix}
       $rules | foreach-object {$oinstance.NormalizationRules += @($_.NormalizationRuleDN)}
       $oInstance.put() | out-null }
}


Filter Add-OCSNormalizationRuleToOCSLocationProfile
{param($Rule,$Location=$(throw "You must specify a Location"), $server=".")
 if ($rule -eq $null) {$rule = $_}
 if ($rule -is [string])    {$rule=Get-OCSnormalizationrule -name "$rule%" -server $server}
 if ($Location -is [string]) {$Location= Get-OCSlocationProfile -name $Location -server $server }
 if ($Location -is [array] ) {$location | forEach {Link-OCSNormalizationRuleToLocation -rules $rule -Location $_ -Server $server} }
 if (($Location -is [System.Management.ManagementObject]) -and ($rule -is [System.Management.ManagementObject]) ) {$rule | foreach-object {$Location.NormalizationRules=$Location.NormalizationRules + $_.NormalizationRuleDN} 
                                                           $location.put() | out-null}
 $rule = $null
}

set-alias -name Add-OCSRuleToLocation  -value Add-OCSNormalizationRuleToOCSLocationProfile

Filter Remove-OCSNormalizationRuleFromOCSLocationProfile
{param($Rule=$(throw "You must specify a Rule"),$Location="%", $server=".")
 if ($Rule -is [string])    {$rule=Get-OCSnormalizationrule -name $rule -server $server}
 if ($rule -is [System.Management.ManagementObject] ) { 
     if ($Location -is [string]) {$Location= Get-OCSlocationProfile -name $Location -server $server | where { $_.NormalizationRules -contains $rule.NormalizationRuleDN } }
     if (($Location -is [System.Management.ManagementObject]) -Or ($Location -is [array] ) ){
	forEach ($l in $location) {$l.NormalizationRules | foreach -Begin {write-host "Updating $($l.name)"; $ruleList=@()} `
                                                                -Process {if ($_ -ne $Rule.NormalizationRuleDN) {$RuleList=$ruleList + $_} }`
                                                                -end {$l.normalizationRules = $ruleList; $l.put()} | Out-Null }
     }}
}

Set-alias -name Remove-OcsRuleFromLocation  -value Remove-OCSNormalizationRuleFromOCSLocationProfile


Function Get-OCSPhoneRoute 
{Param($name="%", $Server=".") 
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPPhoneRouteData where name like '$name' " -computername $server | sort-object -property Name }


Function Choose-OCSPhoneRoute {Param($Server=".") ;  choose-list -input (Get-OCSPhoneRoute -server $Server) -property name,description -multi }


Function Remove-OCSPhoneRoute 
{Param([String]$Name=$(throw "You must specify a name for the route"), $Server=".") 
 (Get-OCSPhoneRoute -Server $Server -name $name).delete() 
}


Function New-OCSPhoneRoute 
{Param([String]$Name=$(throw "You must specify a name for the route") , $GatewayList=$(throw "You must specify one or more Gateways"), 
       $PhoneRouteUsages=$(throw "You must specify one or more usages") , $TargetPhoneNumber=$(throw "You must specifiy a target phone number expression"), $description, $Server=".")
 if (Get-OCSPhoneRoute -name $name -Server $Server) {return $name + " already exists"}
 else {$oInstance=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPPhoneRouteData").createinstance()
       $oInstance.Name=$name
       $oInstance.description=$description
       $oInstance.TargetPhoneNumber=$TargetPhoneNumber
       $GatewayList | foreach-object {if ($_ -is  [System.Management.ManagementBaseObject]) 
                                         {$oInstance.gatewayList=$oInstance.gatewayList + ($_.fqdn +":"+$_.port)}
		                     elseif ($_ -is  [String]) 
                                         {$oInstance.gatewayList=$oInstance.gatewayList + $_ }}
       $PhoneRouteUsages |  foreach-object {if ($_ -is  [System.Management.ManagementBaseObject]) 
                                         {$oInstance.PhoneRouteUsages=$oInstance.PhoneRouteUsages + ($_.PhoneRouteUsageDN)}
	                             elseif ($_ -is  [String]) 
                                         {$oInstance.PhoneRouteUsages=$oInstance.PhoneRouteUsages + $_ }}

       $oInstance.put() | out-null}
}


Filter Update-OCSPhoneRoute 
{Param($Name, $TargetPhoneNumber, $description, $GatewayList, $PhoneRouteUsages, $Server=".")
 if ($name) {$oInstance=Get-OCSPhoneRoute -name $name -Server $Server} else {$oInstance=$_}
 if ($oInstance) {
       if ($description)       {$oInstance.description=$description}
       if ($TargetPhoneNumber) {$oInstance.TargetPhoneNumber=$TargetPhoneNumber}
#       if ($GateWayList)
       if ($PhoneRouteUsages)  { $oInstance.PhoneRouteUsages = @()
            $PhoneRouteUsages |  foreach-object {if ($_ -is  [System.Management.ManagementBaseObject])
                                             {$oInstance.PhoneRouteUsages=$oInstance.PhoneRouteUsages + ($_.PhoneRouteUsageDN)}
                                     elseif ($_ -is  [String])
                                             {$oInstance.PhoneRouteUsages=$oInstance.PhoneRouteUsages + $_ }} }
       $oInstance.put() | out-null
       $oInstance}
 else {return "$name  doesn't exist"}
 $oInstance = $null
}



Function Get-OCSPhoneRouteUsage 
{Param($name="%", $Server=".") 
 $name=$name.replace("*","%")
 Get-WmiObject -query "Select * from MSFT_SIPPhoneRouteUsageData where UsageAttribute like '$name' " -computername $server | sort-object -property Name }


Function Choose-OCSPhoneRouteUsage {Param($Server=".") ; choose-list -input (Get-OCSPhoneRouteUsage -server $server) -property @(@{Label="Name"; Expression={$_.usageAttribute}} , "Description" ) -multi }


Function Remove-OCSPhoneRouteUsage
{Param([String]$Name=$(throw "You must specify a name for the usage"), $Server=".") 
 Get-OCSPhoneRouteUsage -Server $Server | where-object {$_.usageAttribute -eq $name} | foreach-object {$_.delete() | out-null}
}


Function New-OCSPhoneRouteUsage
{Param([String]$Name=$(throw "You must specify a name for the Usage") , $description, $Server=".")
 if (Get-OCSPhoneRouteUsage -name $name -Server $Server) {return $name + " already exists"}
 else {$oInstance=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPPhoneRouteUsageData").createinstance()
       $oInstance.UsageAttribute=$name
       $oInstance.description=$description
       $oInstance.defaultUsage=$false
       $oInstance.put() | out-null }
}


Function Get-OCSGlobalUCSetting  
{Param($Server=".")
 Get-WmiObject -class msft_sipGlobalUCSetting -computername $server 
}


Filter Get-OCSUCPolicyForOCSUser 
{Param($User, $Server="." )
 if ($user -eq $null) {$user = $_ }
 if ($user -is [string]) {$User=Get-OCSuser  -uri $User  -server $server }
 if ($user -is [array]) {$user | foreach {Get-OCSUCPolicyForOCSUser -user $_ -Server $Server} }
 if ($user -is [System.Management.ManagementBaseObject]) { $GS= Get-OCSGlobalUCSetting  
                                                           if (($GS.UsePerUserUCPolicy -eq $false) -or ($user.ucpolicy -eq $null)) {$pol=$gs.UCPolicy}
                                                           else {$pol= $user.UCPolicy}
                                                           Get-wmiObject -query  "Select * from  MSFT_SIPGlobalUCPolicyData where PolicyDN='$pol'"}
 $user=$nul
}


Filter Get-OCSPhoneRouteUsageForOCSUCPolicy
{param($Policy , $Server="." ) 
 if   ($Policy -eq $null) {$policy = $_}
 if   ($Policy -is [string]) {$Policy=Get-OCSUCPolicy  -name $Policy  -server $server }
 if   ($Policy -is [array]) {$Policy | foreach {Get-OCSPhoneRouteUsageForOCSUCPolicy -Policy $_ -Server $Server} }
 if ($Policy -is [System.Management.ManagementBaseObject]) {$Policy.phonerouteUsages | foreach-object {Get-wmiObject -computerName $Policy.__Server -query  "Select * from  MSFT_SIPPhoneRouteUsageData where PhoneRouteUsageDN='$_'"}} 
 $Policy=$null
}


Filter Get-OCSPhoneRouteForOCSPhoneRouteUsage 
{param($usages, $Server=".") 
 if ($usages -is [string]) {$usages=Get-OCSPhoneRouteUsage -Server $server -name $usages }
 if ($usages -eq $null)    {$usages=$_}  
 if ($usages) {foreach ($usage in $usages) {$usage.PhoneRoutes | foreach-object {Get-wmiObject  -ComputerName $Usage.__Server -query  "Select * from  MSFT_SIPPhoneRoutedata where PhoneRouteDN='$_'"}}}
 $usages=$null
} 

set-alias -name Get-OCSRouteForUsage  -value Get-OCSPhoneRouteForOCSPhoneRouteUsage

Filter Get-OCSPhoneRouteForOCSUser
{Param($user, $Server=".")
 if ($user -eq $null) {$user=$_}
 Get-OCSUCPolicyForOCSUser -user $user -server $server  | Get-OCSPhoneRouteUsageForOCSUCPolicy  |  Get-OCSPhoneRouteForOCSPhoneRouteUsage | sort-object -property name -unique
 $user=$null
}

set-alias Get-OCSRouteForUser -value Get-OCSPhoneRouteForOCSUser

Filter Get-OCSPhoneRouteForNumber
{Param([String]$Number=$(throw "You must specify a Number"), $routes=$(Get-OCSPhoneRoute))
 $Matches = $null
 $routes | where-object {$number -match $_.TargetPhoneNumber}
 if ($matches -eq $null) {"No matches found for $number"} 
}


Function Add-OCSPhoneRouteUsageToOCSPhoneRoute
#it may seem more logical to do this the other way round, but wmi doesn't seem to let you add the route to the usage ... 
{param($Usages=$(throw "You must specify one or more usages"),$Route=$(throw "You must specify a route"), $server=".")
 if ($Usages -is [string]) {$Usages=Get-OCSphonerouteUsage -name "$usages%" -server $Server }
 if ($Route -is [string])  {$route=Get-OCSphoneroute -name $route -server $server}
 if ($route -is [Array] )  {$route | foreach-object { Add-OCSPhoneRouteUsageToOCSPhoneRoute -route $_ -usages $Usages } }
 if ($route -is [System.Management.ManagementObject]) { $usages | foreach-object {if ($Route.PhoneRouteUsages -notcontains  $_.PhoneRouteUsageDN ) {$Route.PhoneRouteUsages=$Route.PhoneRouteUsages +  $_.PhoneRouteUsageDN}}
                                                        $route.put() | out-Null}
}

Set-Alias -name Add-OCSUsageTORoute -value Add-OCSPhoneRouteUsageToOCSPhoneRoute


Function Remove-OCSPhoneRouteUsageFromOCSPhoneRoute
{param($Usage=$(throw "You must specify a Usage"),$Route="%", $server=".")
 if ($Usage -is [string])    {$Usage=Get-OCSPhoneRouteUsage -name $Usage -server $server}
 if ($Usage -is [System.Management.ManagementObject] ) { 
     if ($Route -is [string]) {$Route= Get-OCSPhoneRoute -name $Route -server $server | where { $_.PhoneRouteUsages -contains $Usage.PhoneRouteUsageDN } }
     if (($Route -is [System.Management.ManagementObject]) -Or ($Route -is [array] ) ){
	forEach ($R in $Route) {$R.PhoneRouteUsages | foreach -Begin {write-host "Updating $($r.name)"; $UsageList=@()} `
                                                                -Process {if ($_ -ne $Usage.PhoneRouteUsageDN) {$UsageList=$UsageList + $_} }`
                                                                -end {$R.PhoneRouteUsages = $UsageList; $r.put()} | Out-Null }
     }}
}

Set-Alias -name Remove-OCSUsageFromRoute -value Remove-OCSPhoneRouteUsageFromOCSPhoneRoute


Function Add-OCSPhoneRouteUsageToOCSUCPolicy
{param($Usages=$(throw "You must specify one or more usages"),$Policy=$(throw "You must specify a Policy"), $server=".")
 if ($Usages -is [string]) {$Usages=Get-OCSphonerouteUsage -name "$usages%" -server $Server }
 if ($Policy -is [string]) {$Policy=Get-OCSucpolicy -name $policy -Server $Server}
 if ($policy -is [Array])  {$Policy | ForEach {Add-OCSPhoneRouteUsageToOCSUCPolicy -Policy $_ -usages $usages} }
 if ($Policy -is [System.Management.ManagementObject]) {  $usages | foreach-object {If ($Policy.phoneRouteUsages -notcontains  $_.PhoneRouteUsageDN) {$Policy.PhoneRouteUsages=$Policy.PhoneRouteUsages +  $_.PhoneRouteUsageDN}}
                                                           $Policy.put() | out-Null }
}



Function Remove-OCSPhoneRouteUsageFromOCSUCPolicy
{param($Usage=$(throw "You must specify a Usage"),$Policy="%", $server=".")
 if ($Usage -is [string])    {$Usage=Get-OCSPhoneRouteUsage -name $Usage -server $server}
 if ($Usage -is [System.Management.ManagementObject] ) { 
     if ($Policy -is [string]) {$Policy= Get-OCSUCPolicy -name $Policy -server $server | where { $_.PhoneRouteUsages -contains $Usage.PhoneRouteUsageDN } }
     if (($Policy -is [System.Management.ManagementObject]) -Or ($Policy -is [array] ) ){
	forEach ($P in $Policy) {$P.PhoneRouteUsages | foreach -Begin {write-host "Updating $($P.name)"; $UsageList=@()} `
                                                                -Process {if ($_ -ne $Usage.PhoneRouteUsageDN) {$UsageList=$UsageList + $_} }`
                                                                -end {$P.PhoneRouteUsages = $UsageList; $P.put()} | Out-Null }
     }}
}




Function Start-OCSReplication
{Param($pool=$(throw "You must specify a pool or SE server"))
 $dbPath=(Get-OCSpool -name $Pool).backendDbpath.replace("\","\\")
 $instance=(Get-wmiObject -q "select * from MSFT_SIPUserReplicatorSetting where backend='$dbPath' " -computerName $pool)
 $instance.RegenerateCookiesNow=$true
 $instance.Put() | Out-null
}


Function Get-OCSWindowsService {Param($Server=".") ;  Get-WmiObject -query "select * from win32_service where name like 'RTC%' " -computername $server | sort-object -property displayname } 


Function Stop-OCSWindowsService
{Param($Server=".")
 Get-OCSWindowsService -Server $Server | foreach-object {$_.StopService() | out-null }
 Get-OCSWindowsService -Server $Server | sort-object -property displayname | format-table  -autosize -wrap -property DisplayName, State, Status, StartMode  
}


Function Start-OCSWindowsService
{Param($Server=".")
 Get-OCSWindowsService -Server $Server | foreach-object {$_.StartService() | out-null}
 Get-OCSWindowsService -Server $Server | sort-object -property displayname | format-table  -autosize -wrap -property DisplayName, State, Status, StartMode  
}


Function Get-OCSSipRoutingCert
{$sn=''
 (Get-WmiObject -class MSFT_SIPRoutingSetting).tlsCertSN | foreach { if ($_  -lt 16) {"0{0:x}" -f $_} else {"{0:x}" -f $_} } | foreach { $sn = $_ + $SN  }
 dir cert:\localmachine\my | where {$_.serialnumber -eq $sn} | format-list -property FriendlyName,Subject, Issuer, NotBefore, NotAfter,
   @{label="Enhanced key Usage"; expression={$_.Extensions | foreach { if ($_ -is [System.Security.Cryptography.X509Certificates.X509EnhancedKeyUsageExtension]) {$_.EnhancedKeyUsages|foreach {$_.FriendlyName} }  } }}
}
#works for mediation servers


Function Get-OCSEdgeFederationExternalCert
{$sn=''
 (Get-WmiObject -class MSFT_SIPFederationExternalEdgeListeningAddressSetting).tlsCertSN | foreach { if ($_  -lt 16) {"0{0:x}" -f $_} else {"{0:x}" -f $_} } | foreach { $sn = $_ + $SN  }
 dir cert:\localmachine\my | where {$_.serialnumber -eq $sn} | format-list -property FriendlyName,Subject, Issuer, NotBefore, NotAfter,
   @{label="Enhanced key Usage"; expression={$_.Extensions | foreach { if ($_ -is [System.Security.Cryptography.X509Certificates.X509EnhancedKeyUsageExtension]) {$_.EnhancedKeyUsages|foreach {$_.FriendlyName} }  } }}
}


Function Get-OCSEdgeFederationInternalCert
{$sn=''
 (Get-WmiObject -class msft_sipfederationInternalEdgeListeningAddressSetting).tlsCertSN | foreach { if ($_  -lt 16) {"0{0:x}" -f $_} else {"{0:x}" -f $_} } | foreach { $sn = $_ + $SN  }
 dir cert:\localmachine\my | where {$_.serialnumber -eq $sn} | format-list -property FriendlyName,Subject, Issuer, NotBefore, NotAfter,
   @{label="Enhanced key Usage"; expression={$_.Extensions | foreach { if ($_ -is [System.Security.Cryptography.X509Certificates.X509EnhancedKeyUsageExtension]) {$_.EnhancedKeyUsages|foreach {$_.FriendlyName} }  } }}
}


Function Get-OCSEdgeConferencingExternalCert
{$sn=''
 (Get-WmiObject -class msft_sipdataproxysetting).tlsCertSN | foreach { if ($_  -lt 16) {"0{0:x}" -f $_} else {"{0:x}" -f $_} } | foreach { $sn = $_ + $SN  }
 dir cert:\localmachine\my | where {$_.serialnumber -eq $sn} | format-list -property FriendlyName,Subject, Issuer, NotBefore, NotAfter,
   @{label="Enhanced key Usage"; expression={$_.Extensions | foreach { if ($_ -is [System.Security.Cryptography.X509Certificates.X509EnhancedKeyUsageExtension]) {$_.EnhancedKeyUsages|foreach {$_.FriendlyName} }  } }}
}


Function Get-OCSEdgeAvAuthCert
{$sn=''
 (Get-WmiObject -class msft_sipMediaRelaySetting).BankCertSN | foreach { if ($_  -lt 16) {"0{0:x}" -f $_} else {"{0:x}" -f $_} } | foreach { $sn = $_ + $SN  }
 dir cert:\localmachine\my | where {$_.serialnumber -eq $sn} | format-list -property FriendlyName,Subject, Issuer, NotBefore, NotAfter,
   @{label="Enhanced key Usage"; expression={$_.Extensions | foreach { if ($_ -is [System.Security.Cryptography.X509Certificates.X509EnhancedKeyUsageExtension]) {$_.EnhancedKeyUsages|foreach {$_.FriendlyName} }  } }}
}


Function Get-OCSEdgeInternalDomain {Param($Server=".") ;  Get-wmiobject -class MSFT_SIPFederationInternalDomainData -computername $server }


Function New-OCSEdgeInternalDomain
{Param([String]$Domain=$(throw "You must specify a Domain"), $Server=".")
 $oInstance=([wmiclass]"\\$Server\root\cimv2:msft_sipFederationInternalDomainData").createinstance()
 $oInstance.supportedinternaldomain = $Domain
 $oInstance.put() | out-null
}


Function Remove-OCSEdgeInternalDomain
{Param([String]$Domain=$(throw "You must specify a Domain"), $Server=".") 
 (Get-wmiobject -computername $server  -query "Select * from msft_sipFederationInternalDomainData where supportedinternaldomain='$domain' ").delete() | out-null 
}


Function Get-OCSEdgeInternalServer {Param($Server=".") ;  Get-wmiobject -class MSFT_SIPFederationInternalServerData -computername $server}


Function New-OCSEdgeInternalServer
{Param([String]$IntServer=$(throw "You must specify an interal Server"),$server=".")
 $oInstance=([wmiclass]"\\$server\root\cimv2:MSFT_SIPFederationInternalServerData").createinstance()
 $oInstance.InternalServer= $IntServer
 $oInstance.put() | out-null
}


Function Remove-OCSEdgeInternalServer
{Param([String]$IntServer=$(throw "You must specify an internal Server"), $Server=".")
 (Get-wmiobject -computername $server  -query "Select * from MSFT_SIPFederationInternalServerData where InternalServer='$IntServer' ").delete() | out-null
}


Function Get-OCSEdgeFederationDenied {Param($Server=".");  (Get-wmiobject -class msft_sipFederationDeniedDomainSetting -computername $server) }


Function Remove-OCSEdgeFederationDenied
{Param([String]$Domain=$(throw "You must specify a Domain"), $Server=".")
 $fd=(Get-wmiobject -class msft_sipFederationDeniedDomainSetting -computername $server)
 If ($fd.deniedDomain -contains $domain) {$fd.denieddomain | foreach {if ($_ -ne $domain) {[string[]]$dd=$dd + $_} }
     $fd.denieddomain = $dd
     $fd.put() | out-null}
}


Function New-OCSEdgeFederationDenied
{Param([String]$Domain=$(throw "You must specify a Domain"), $Server="."  )
 $fd=(Get-wmiobject -class msft_sipFederationDeniedDomainSetting -computername $server)
 If ( !($fd.deniedDomain -contains $domain)) {$fd.denieddomain= $fd.denieddomain + $domain
 $fd.put() | out-null}
}


Function Get-OCSEdgeFederationPartner {Param($Server=".") ; Get-WmiObject -class MSFT_SIPFederationPartnerTable -computername $server }


Function New-OCSEdgeFederationPartner
{Param([String]$Domain=$(throw "You must specify a Domain") ,$edgeProxyAddress, $UserVerificationLevel=$("UseSourceVerification"), $server=".")
 $oInstance=([wmiclass]"\\$Server\root\cimv2:MSFT_SIPFederationPartnerTable").createinstance()
 $oInstance.Domain = $Domain
 $oInstance.EdgeProxyAddress = $EdgeProxyAddress
 $oInstance.UserVerificationLevel= $UserVerificationLevel
 $oInstance.put() | out-null
}


Function Export-OCSEdgeFederationPartner {Param( [String]$Path=$(throw "You must specify a file"), $Server=".") ;  Get-OCSEdgeFederationPartner -Server $Server | Select-object Domain, EdgeProxyAddress | Export-csv $path }


Function Import-OCSEdgeFederationPartner {Param([String]$Path=$(throw "You must specify a file"), $Server=".") ;  Import-csv $path | foreach {New-OCSEdgeFederationPartner -domain $_.domain -EdgeProxyAddress $_.edgeProxyAddress -Server $Server} }


Function Remove-OCSEdgeFederationPartner
{Param([String]$Domain=$(throw "You must specify a Domain") , $Server=".")
 (Get-WmiObject -computername $server  -query "Select * from MSFT_SIPFederationPartnerTable  where Domain='$Domain'").delete() | out-null
} 

#to delete all the providers and re-import them: Get-OCSEdgeFederationPartner | foreach {Remove-OCSEdgeFederationPartner -domain $_.domain}


Function Get-OCSEdgeIMProvider
{Param($Server=".")
 Get-WmiObject -class MSFT_SIPFederationNetworkProviderTable -computername $server
}


Function New-OCSEdgeIMProvider
 {Param([String]$Name=$(throw "You must specify a name for the provider"),
        [String]$edgeProxyAddress=$(throw "You must specify an EdgeProxy Address"),
        [String]$UserVerificationLevel=$("UseSourceVerification"),$server=".",[switch]$Enabled,[Switch]$IsPublic)
  $oInstance=([wmiclass]"\\$server\root\cimv2:MSFT_SIPFederationNetworkProviderTable").createinstance()
  $oInstance.NetworkProviderName = $name
  $oInstance.EdgeProxyAddress = $EdgeProxyAddress
  $oInstance.UserVerificationLevel=$UserVerificationLevel
  $oInstance.Enabled = $Enabled.ispresent
  $oInstance.IsPublicNetworkProvider = $IsPublic.ispresent
  $oInstance.put() | out-null
}


Function Update-OCSEdgeIMProvider
{Param([String]$Name=$(throw "You must specify a name for the provider"), [String]$edgeProxyAddress, [String]$UserVerificationLevel, $server="." , [switch]$Enabled,[Switch]$IsPublic)
 $oInstance=Get-WmiObject -computername $server  -query "Select * from MSFT_SIPFederationNetworkProviderTable where NetworkProviderName='$Name'"
 if ($EdgeProxyAddress -ne '') {$oInstance.EdgeProxyAddress = $EdgeProxyAddress}
 if($UserVerificationLevel -ne '') {$oInstance.UserVerificationLevel=$UserVerificationLevel}
 $oInstance.Enabled = $Enabled.ispresent
 $oInstance.IsPublicNetworkProvider = $IsPublic.ispresent
 $oInstance.put() | out-null
}


Function Remove-OCSEdgeIMProvider
{Param( [String]$Name=$(throw "You must specify a Name for the provider"), $Server=".")
 (Get-WmiObject -computername $server  -query "Select * from MSFT_SIPFederationNetworkProviderTable where NetworkProviderName='$Name'").delete()
} 


Function Get-OCSTrustedService 
{Param($Server=".", $type="%") 
 Get-WmiObject -query "Select * from MSFT_SIPTrustedServiceSetting where type like '$Type' " -computername $server   |  sort-object -property FQDN }


Function Choose-OCSMediationServer {Param($Server=".");  choose-list -input (Get-OCSTrustedService -server $server -type 'MediationServer') -property fqdn,port - multi }


Function Get-OCSMediationServerSetting {Param($Server=".") ;  Get-WmiObject -class MSFT_SIPMediationServerConfigSetting -computername $server  }


Filter Set-OCSMediationServerPlusSign 
{param($Server , [Switch]$Remove,  [Switch]$Preserve)
 if ($server -eq $null) {if ($_) {$server=$_.fqdn} else {$server="."}  }
 if ($preserve) {$newValue = $false}
 elseif ($remove) {$newValue=$True} else {$newVlaue=$False}
 $setting=Get-wmiObject -computerName $server -class MSFT_SIPMediationServerConfigSetting
 $setting.RemovePlusFromRequestURI = $NewValue
 $setting.put() | out-null
 $server=$null
}

Function Enable-ExchUmForOcsUser
{Param ($filter, $exchUmPolicy, [Switch]$runNow)
 $UCEnabledUsers=Get-aduser -filter "(&(msRTCSIP-Line=*)(msrtcsip-userlocationprofile=*)(msexchHomeServerName=*)$filter)" | where {($_.Properties.Item("msrtcsip-optionflags")[0]  -band 128) -and ($_.Properties.Item("msexchumrecipientdialplanlink").Count -eq 0)}
 foreach ($u in $UCEnabledUsers) {
   $l=([adsi]("LDAP://"+$u.Properties.Item("msrtcsip-userlocationprofile")[0])).psbase.properties.get_Item("msRtcSIP-LocationName")
   $d= Get-ExchUMDialPlan $l 
   $Number= $u.Properties.item("msrtcsip-line")[0]
   $Ext=$number.substring( $number.length - $d.Properties.Item("msexchumnumberingplandigits")[0] )
  
   if ($ExchUMPolicy -eq $null) {$UCPolicies=$d.Properties.Item("msexchummailboxpolicydialplanbl")[0] | foreach {([adsi]"LDAP://$_")}
                               $ExchUMPolicy= (choose-list -input $UCPolicies -prop name ).name }
  
    $command = "Enable-UMMailbox  -Identity $($u.properties.item('mail'))  -PinExpired `$false -UMMailboxPolicy '$ExchUmPolicy' -Extensions '$ext' -SIPResourceIdentifier '$($u.properties.item('msrtcsip-primaryuseraddress')[0].subString(4))'"
    $command
  if ($runNow) {invoke-expression $command}
 }
}


Function Get-ExchUMDialPlan
{Param($name="*")
 #runs against default AD server
 $searcher= New-Object directoryServices.DirectorySearcher([adsi]('LDAP://'+([adsi]'LDAP://RootDse').configurationNamingContext))
 $searcher.filter = "(&(objectclass=msExchUMDialPlan)(cn=$name)(msExchUMDialPlanVoipSecurity=1)(msExchUMDialPlanURIType=3))"
 $searcher.Findall()
}


Function Get-OCSFunction {dir Function:*OCS* | sort @{expression={$_.name.split("-")[1] + $_.name.split("-")[0] }} | Select-Object name}


if ($myinvocation.line -match "^\.\s") {
   dir Function:*OCS* | measure-object | format-table -property count, @{expression={"OCS Functions loaded"} } -HideTableHeaders -autosize
   "Use Get-OCSFunction to get a list"}
Else {write-host -ForegroundColor red "No functions were loaded - you need to invoke with . scriptname "}

Update-FormatData  (Join-Path (Get-scriptPath) "OCSTypes.format.ps1xml")

